@@ -1,13 +1,15 @@
 import logging
 from typing import Union
 from collections import namedtuple, defaultdict
-from math import sqrt
+from math import sqrt, floor
 
 import requests
 from django.core.cache import caches
-from django.utils.translation import ugettext as _
+from django.utils.translation import gettext as _
 from django.conf import settings
 
+from observation_portal.common.utils import cache_function
+
 logger = logging.getLogger(__name__)
 
 
@@ -16,7 +18,7 @@ class ConfigDBException(Exception):
     pass
 
 
-class TelescopeKey(namedtuple('TelescopeKey', ['site', 'enclosure', 'telescope'])):
+class TelescopeKey(namedtuple('TelescopeKey', ['site', 'enclosure', 'telescope', 'telescope_class'])):
     """Generate the key of a telescope."""
     __slots__ = ()
 
@@ -28,6 +30,7 @@ class ConfigDB(object):
     """Class to retrieve and process configuration data."""
 
     @staticmethod
+    @cache_function(duration=900)
     def _get_configdb_data(resource: str):
         """Return all configuration data.
 
@@ -43,21 +46,16 @@ class ConfigDB(object):
             'ConfigDB connection is currently down, please wait a few minutes and try again. If this problem '
             'persists then please contact support.'
         ))
-        data = caches['locmem'].get(resource)
-        if not data:
-            try:
-                r = requests.get(settings.CONFIGDB_URL + f'/{resource}/')
-                r.raise_for_status()
-            except (requests.exceptions.RequestException, requests.exceptions.HTTPError) as e:
-                msg = f'{e.__class__.__name__}: {error_message}'
-                raise ConfigDBException(msg)
-            try:
-                data = r.json()['results']
-            except KeyError:
-                raise ConfigDBException(error_message)
-            # Cache the results for 15 minutes.
-            caches['locmem'].set(resource, data, 900)
-        return data
+        try:
+            r = requests.get(settings.CONFIGDB_URL + f'/{resource}/')
+            r.raise_for_status()
+        except (requests.exceptions.RequestException, requests.exceptions.HTTPError) as e:
+            msg = f'{e.__class__.__name__}: {error_message}'
+            raise ConfigDBException(msg)
+        try:
+            return r.json()['results']
+        except KeyError:
+            raise ConfigDBException(error_message)
 
     def get_site_data(self):
         """Return ConfigDB sites data."""
@@ -123,12 +121,37 @@ class ConfigDB(object):
             telescopes.append(('', ''))
         return telescopes
 
+    def convert_telescope_aperture_to_string(self, aperture):
+        ''' This takes in a float aperture and converts it to a string of the form #m#
+            where the first # is the left side of the decimal point, and the second number
+            is the rounded right side of the decimal.
+        '''
+        left_side = floor(aperture)
+        right_side = round((aperture - left_side) * 10.0)
+        return f'{left_side}m{right_side}'
+
+    @cache_function()
+    def get_telescope_key(self, site_code='', enclosure_code='', telescope_code=''):
+        for site in self.get_site_data():
+            if not site_code or site['code'].lower() == site_code.lower():
+                for enclosure in site['enclosure_set']:
+                    if not enclosure_code or enclosure['code'].lower() == enclosure_code.lower():
+                        for telescope in enclosure['telescope_set']:
+                            if not telescope_code or telescope['code'].lower() == telescope_code.lower():
+                                return TelescopeKey(
+                                    site=site_code,
+                                    enclosure=enclosure_code,
+                                    telescope=telescope_code,
+                                    telescope_class=self.convert_telescope_aperture_to_string(telescope['aperture'])
+                                )
+        return TelescopeKey(site=site_code, enclosure=enclosure_code, telescope=telescope_code, telescope_class='N/A')
+
     def get_telescope_class_tuples(self):
         telescope_classes = set()
         for site in self.get_site_data():
             for enclosure in site['enclosure_set']:
                 for telescope in enclosure['telescope_set']:
-                    telescope_classes.add(telescope['code'][:-1])
+                    telescope_classes.add(self.convert_telescope_aperture_to_string(telescope['aperture']))
         return [(telescope_class, telescope_class) for telescope_class in telescope_classes]
 
     def get_telescope_name_tuples(self):
@@ -188,7 +211,7 @@ class ConfigDB(object):
                                     ):
                                         instrument_names.add(instrument['code'].lower())
                                         instrument_types.add(
-                                            instrument['instrument_type']['code'].lower()
+                                            instrument['instrument_type']['code'].upper()
                                         )
         return {'names': instrument_names, 'types': instrument_types}
 
@@ -255,7 +278,8 @@ class ConfigDB(object):
                             telescope_key = TelescopeKey(
                                 site=site['code'],
                                 enclosure=enclosure['code'],
-                                telescope=telescope['code']
+                                telescope=telescope['code'],
+                                telescope_class=self.convert_telescope_aperture_to_string(telescope['aperture'])
                             )
                             instrument['telescope_key'] = telescope_key
                             instrument['telescope_name'] = telescope['name'].strip().lower()
@@ -290,11 +314,10 @@ class ConfigDB(object):
             exclude_states = ['DISABLED', 'ENABLED', 'MANUAL', 'COMMISSIONING', 'STANDBY']
         telescope_instrument_types = {}
         for instrument in self.get_instruments(exclude_states=exclude_states):
-            split_string = instrument['__str__'].lower().split('.')
-            if (location.get('site', '').lower() in split_string[0]
-                    and location.get('enclosure', '').lower() in split_string[1]
-                    and location.get('telescope_class', '').lower() in split_string[2]
-                    and location.get('telescope', '').lower() in split_string[2]):
+            if (location.get('site', '').lower() in instrument['telescope_key'].site
+                    and location.get('enclosure', '').lower() in instrument['telescope_key'].enclosure
+                    and location.get('telescope_class', '').lower() in instrument['telescope_key'].telescope_class
+                    and location.get('telescope', '').lower() in instrument['telescope_key'].telescope):
                 if instrument['telescope_key'] not in telescope_instrument_types:
                     telescope_instrument_types[instrument['telescope_key']] = []
                 instrument_type_code = instrument['instrument_type']['code'].upper()
@@ -321,7 +344,7 @@ class ConfigDB(object):
                     instrument['telescope_key'].site.lower() == site_code.lower()
                     and instrument['telescope_key'].enclosure.lower() == enclosure_code.lower()
                     and instrument['telescope_key'].telescope.lower() == telescope_code.lower()
-                    and instrument['instrument_type']['code'].lower() == instrument_type_code.lower()
+                    and instrument['instrument_type']['code'].upper() == instrument_type_code.upper()
             ):
                 instrument_names.add(instrument['code'].lower())
         return instrument_names
@@ -356,10 +379,11 @@ class ConfigDB(object):
             exclude_states = ['DISABLED', 'ENABLED', 'MANUAL', 'COMMISSIONING', 'STANDBY']
         instrument_telescopes = set()
         for instrument in self.get_instruments(exclude_states=exclude_states):
-            if instrument['instrument_type']['code'].upper() == instrument_type_code:
+            if instrument['instrument_type']['code'].upper() == instrument_type_code.upper():
                 instrument_telescopes.add(instrument['telescope_key'])
         return instrument_telescopes
 
+    @cache_function()
     def get_configuration_types(self, instrument_type_code: str) -> dict:
         """Get the available configuration types for an instrument_type.
 
@@ -373,6 +397,21 @@ class ConfigDB(object):
             return {config_type['code']: config_type for config_type in instrument_types[instrument_type_code.upper()]['configuration_types']}
         return {}
 
+    @cache_function()
+    def get_default_configuration_type(self, instrument_type_code: str) -> str:
+        """Get the default configuration types for an instrument_type is it exists.
+
+        Parameters:
+            instrument_type_code: Instrument type for which to get the configuration types
+        Returns:
+             str code of default configuration type or empty string
+        """
+        instrument_types = self.get_instrument_types()
+        if instrument_type_code.upper() in instrument_types and instrument_types[instrument_type_code.upper()]['default_configuration_type']:
+            return instrument_types[instrument_type_code.upper()]['default_configuration_type']
+        return ''
+
+    @cache_function()
     def get_optical_elements(self, instrument_type_code: str) -> dict:
         """Get the available optical elements.
 
@@ -389,7 +428,7 @@ class ConfigDB(object):
                     for optical_element_group in science_camera['optical_element_groups']:
                         for element in optical_element_group['optical_elements']:
                             if element['code'] not in optical_elements_tracker[optical_element_group['type']]:
-                                if optical_element_group['default'].upper() == element['code'].upper():
+                                if optical_element_group['default'].lower() == element['code'].lower():
                                     element['default'] = True
                                 else:
                                     element['default'] = False
@@ -397,6 +436,7 @@ class ConfigDB(object):
                                 optical_elements[optical_element_group['type']].append(element)
         return optical_elements
 
+    @cache_function()
     def get_modes_by_type(self, instrument_type_code: str, mode_type: str = '') -> dict:
         """Get the set of available modes.
 
@@ -419,6 +459,7 @@ class ConfigDB(object):
                             return {mode_type: mode_group}
         return {}
 
+    @cache_function()
     def get_instrument_type_by_code(self, instrument_type_code: str) -> dict:
         """Get an instrument type by its code.
 
@@ -433,6 +474,7 @@ class ConfigDB(object):
 
         raise ConfigDBException(f'No instrument type found for instrument type code {instrument_type_code}')
 
+    @cache_function()
     def get_mode_with_code(self, instrument_type, code, mode_type=''):
         modes_by_type = self.get_modes_by_type(instrument_type, mode_type)
         for _, mode_group in modes_by_type.items():
@@ -514,9 +556,10 @@ class ConfigDB(object):
     def get_instrument_type_telescope_class(self, instrument_type_code: str) -> str:
         for instrument in self.get_instruments():
             if instrument_type_code.upper() == instrument['instrument_type']['code'].upper():
-                return instrument['__str__'].split('.')[2][0:3]
-        return instrument_type_code[0:3]
+                return instrument['telescope_key'].telescope_class
+        return 'None'
 
+    @cache_function()
     def get_instrument_type_codes(self, location: dict, only_schedulable: bool = False) -> set:
         """Get the available instrument_types.
 
@@ -530,11 +573,10 @@ class ConfigDB(object):
         if only_schedulable:
             exclude_states = ['DISABLED', 'ENABLED', 'MANUAL', 'COMMISSIONING', 'STANDBY']
         for instrument in self.get_instruments(exclude_states=exclude_states):
-            split_string = instrument['__str__'].lower().split('.')
-            if (location.get('site', '').lower() in split_string[0]
-                    and location.get('enclosure', '').lower() in split_string[1]
-                    and location.get('telescope_class', '').lower() in split_string[2]
-                    and location.get('telescope', '').lower() in split_string[2]):
+            if (location.get('site', '').lower() in instrument['telescope_key'].site
+                    and location.get('enclosure', '') in instrument['telescope_key'].enclosure
+                    and location.get('telescope_class', '').lower() in instrument['telescope_key'].telescope_class
+                    and location.get('telescope', '').lower() in instrument['telescope_key'].telescope):
                 instrument_types.add(instrument['instrument_type']['code'].upper())
         return instrument_types
 
@@ -555,6 +597,7 @@ class ConfigDB(object):
                     return True
         return False
 
+    @cache_function()
     def get_exposure_overhead(self, instrument_type_code, readout_mode):
         # using the instrument type code, build an instrument with the correct configdb parameters
         for instrument in self.get_instruments():
@@ -573,9 +616,9 @@ class ConfigDB(object):
             if not default_mode:
                 raise ConfigDBException(f'No readout mode named {readout_mode} found for instrument type {instrument_type_code} and no default readout mode set.')
             return default_mode['overhead'] + instrument_type['fixed_overhead_per_exposure']
-
         raise ConfigDBException(f'Instruments of type {instrument_type_code} not found in configdb.')
 
+    @cache_function()
     def get_request_overheads(self, instrument_type_code: str) -> dict:
         """Get the set of overheads needed to compute the duration of a request.
 
