@@ -3,11 +3,11 @@ from mixer.backend.django import mixer as dmixer
 from django.utils import timezone
 from datetime import timedelta
 from unittest.mock import patch
-from django.contrib.auth.models import User
 from django.db.models.signals import post_save
 from django.core import mail
 from django_dramatiq.test import DramatiqTestCase
 
+from observation_portal.accounts.test_utils import blend_user
 from observation_portal.observations.signals.handlers import cb_configurationstatus_post_save
 import observation_portal.observations.signals.handlers  # noqa
 import observation_portal.requestgroups.signals.handlers  # noqa
@@ -16,7 +16,6 @@ from observation_portal.common.test_helpers import (
     disconnect_signal
 )
 from observation_portal.proposals.models import Proposal, Membership
-from observation_portal.accounts.models import Profile
 from observation_portal.observations.models import Observation, ConfigurationStatus, Summary
 from observation_portal.requestgroups.models import Request, RequestGroup, Window, Location
 from observation_portal.common.state_changes import (
@@ -30,8 +29,7 @@ class TestStateChanges(SetTimeMixin, TestCase):
     def setUp(self):
         super().setUp()
         self.proposal = dmixer.blend(Proposal)
-        self.user = dmixer.blend(User)
-        dmixer.blend(Profile, user=self.user)
+        self.user = blend_user()
         self.client.force_login(self.user)
         now = timezone.now()
         # Create a requestgroup with 3 requests, each request has 3 configurations with current windows and observations
@@ -347,9 +345,8 @@ class TestStateFromConfigurationStatuses(SetTimeMixin, DramatiqTestCase):
     def setUp(self):
         super().setUp()
         self.proposal = dmixer.blend(Proposal)
-        self.user = dmixer.blend(User)
+        self.user = blend_user(profile_params={'notifications_enabled': True})
         dmixer.blend(Membership, user=self.user, proposal=self.proposal)
-        dmixer.blend(Profile, user=self.user, notifications_enabled=True)
         self.client.force_login(self.user)
         self.now = timezone.now()
         self.window = dmixer.blend(Window, start=self.now - timedelta(days=1), end=self.now + timedelta(days=1))
@@ -444,7 +441,8 @@ class TestStateFromConfigurationStatuses(SetTimeMixin, DramatiqTestCase):
         self.worker.join()
 
         self.assertEqual(len(mail.outbox), 1)
-        self.assertIn(str(self.request.id), str(mail.outbox[0].subject))
+        self.assertIn(f'Request #{self.request.id} has failed', str(mail.outbox[0].subject))
+        self.assertIn('You can disable notifications', str(mail.outbox[0].message()))
         self.assertIn(f"Request url: test_url/{self.request.id}", str(mail.outbox[0].message()))
         self.assertEqual(mail.outbox[0].to, [self.user.email])
 
@@ -543,8 +541,7 @@ class TestRequestState(SetTimeMixin, TestCase):
     def setUp(self):
         super().setUp()
         self.proposal = dmixer.blend(Proposal)
-        self.user = dmixer.blend(User)
-        dmixer.blend(Profile, user=self.user)
+        self.user = blend_user()
         self.client.force_login(self.user)
         self.now = timezone.now()
         self.window = dmixer.blend(Window, start=self.now - timedelta(days=1), end=self.now + timedelta(days=1))
