@@ -2,7 +2,7 @@ from rest_framework import serializers
 from django.utils import timezone
 from django.core.cache import cache
 from django.db import transaction
-from django.utils.translation import ugettext as _
+from django.utils.translation import gettext as _
 from django.utils.module_loading import import_string
 from django.conf import settings
 
@@ -76,14 +76,16 @@ class ConfigurationStatusSerializer(serializers.ModelSerializer):
                               }
                 )
 
+        current_repeat = instance.observation.get_current_repeat(instance.id)
+
         if 'end' in validated_data:
             obs_end_time = validated_data['end']
-            obs_end_time += timedelta(seconds=instance.observation.request.get_remaining_duration(instance.configuration.priority))
+            obs_end_time += timedelta(seconds=instance.observation.request.get_remaining_duration(instance.configuration.priority, current_repeat=current_repeat))
             instance.observation.update_end_time(obs_end_time)
 
         if 'exposures_start_at' in validated_data:
             obs_end_time = validated_data['exposures_start_at']
-            obs_end_time += timedelta(seconds=instance.observation.request.get_remaining_duration(instance.configuration.priority, include_current=True))
+            obs_end_time += timedelta(seconds=instance.observation.request.get_remaining_duration(instance.configuration.priority, include_current=True, current_repeat=current_repeat))
             instance.observation.update_end_time(obs_end_time)
 
         return instance
@@ -191,7 +193,7 @@ class ScheduleSerializer(serializers.ModelSerializer):
             data['site'], data['enclosure'], data['telescope']
         )
         for configuration in data['request']['configurations']:
-            if configuration['instrument_type'].lower() not in allowable_instruments['types']:
+            if configuration['instrument_type'].upper() not in allowable_instruments['types']:
                 raise serializers.ValidationError(_("instrument type {} is not available at {}.{}.{}".format(
                     configuration['instrument_type'], data['site'], data['enclosure'], data['telescope']
                 )))
@@ -355,7 +357,7 @@ class ObservationSerializer(serializers.ModelSerializer):
             data['request'].location.site and data['request'].location.site != data['site'] or
             data['request'].location.enclosure and data['request'].location.enclosure != data['enclosure'] or
             data['request'].location.telescope and data['request'].location.telescope != data['telescope'] or
-            data['request'].location.telescope_class != data['telescope'][0:3]
+            data['request'].location.telescope_class != configdb.get_telescope_key(site_code=data['site'], enclosure_code=data['enclosure'], telescope_code=data['telescope']).telescope_class
         ):
             raise serializers.ValidationError(_('{}.{}.{} does not match the request location'.format(
                 data['site'], data['enclosure'], data['telescope']
@@ -366,7 +368,7 @@ class ObservationSerializer(serializers.ModelSerializer):
             data['site'], data['enclosure'], data['telescope'], only_schedulable=False
         )
         for configuration in data['request'].configurations.all():
-            if configuration.instrument_type.lower() not in available_instruments['types']:
+            if configuration.instrument_type.upper() not in available_instruments['types']:
                 raise serializers.ValidationError(_('Instrument type {} not available at {}.{}.{}'.format(
                     configuration.instrument_type, data['site'], data['enclosure'], data['telescope']
                 )))
@@ -408,3 +410,17 @@ class CancelObservationsSerializer(serializers.Serializer):
             raise serializers.ValidationError("Must include either an observation id list or a start and end time")
 
         return data
+
+
+class CancelObservationsResponseSerializer(serializers.Serializer):
+    canceled = serializers.IntegerField()
+
+
+class LastScheduledSerializer(serializers.Serializer):
+    last_schedule_time = serializers.DateTimeField()
+
+
+# TODO: See if we can get some example data in the docs
+class ObservationFiltersSerializer(serializers.Serializer):
+    fields = serializers.ListField(child=serializers.CharField())
+    choice_fields = serializers.ListField(child=serializers.DictField())
