@@ -8,7 +8,7 @@ from observation_portal.observations.models import Observation
 logger = logging.getLogger(__name__)
 
 
-@dramatiq.actor()
+@dramatiq.actor(time_limit=1800000)
 def delete_old_observations():
     cutoff = timezone.now() - timedelta(days=14)
     logger.info(f'Deleting CANCELED observations before cutoff date {cutoff}')
