@@ -1,13 +1,11 @@
 from django.db import models
 from django.contrib.auth.models import User
-from django.contrib.postgres.fields import JSONField
 from django.utils.functional import cached_property
 from django.core.validators import MinValueValidator, MaxValueValidator
 from django.core.cache import cache
 from django.utils import timezone
 from django.urls import reverse
 from django.forms.models import model_to_dict
-from django.utils.functional import lazy
 from django.utils.module_loading import import_string
 from django.conf import settings
 import logging
@@ -267,6 +265,17 @@ class Request(models.Model):
                   'acceptable to meet the science goal of the Request. Defaults to 100 for FLOYDS observations and '
                   '90 for all other observations.'
     )
+    configuration_repeats = models.PositiveIntegerField(
+        default=1, validators=[MinValueValidator(1)],
+        help_text='The number of times to repeat the set of Configurations in this Request. This is normally used '
+                  'to nod between two or more separate Targets. This field must be set to a value greater than 0.'
+    )
+    extra_params = models.JSONField(
+        default=dict,
+        blank=True,
+        verbose_name='extra parameters',
+        help_text='Extra Request parameters'
+    )
 
     class Meta:
         ordering = ('id',)
@@ -285,7 +294,8 @@ class Request(models.Model):
         cached_duration = cache.get('request_duration_{}'.format(self.id))
         if not cached_duration:
             duration = get_total_request_duration({'configurations': [c.as_dict() for c in self.configurations.all()],
-                                                  'windows': [w.as_dict() for w in self.windows.all()]})
+                                                  'windows': [w.as_dict() for w in self.windows.all()],
+                                                  'configuration_repeats': self.configuration_repeats})
             cache.set('request_duration_{}'.format(self.id), duration, 86400 * 30 * 6)
             return duration
         else:
@@ -301,7 +311,19 @@ class Request(models.Model):
 
     @property
     def semester(self):
-        return get_semester_in(self.min_window_time, self.max_window_time)
+        # Get the semester that contains the request windows. This should return a semester since requests are validated on
+        # submission to have all windows be in the same semester, but some old requests might have windows that span multiple semesters.
+        semester = get_semester_in(self.min_window_time, self.max_window_time)
+        # If the request windows do not fit within any single semester, use the semester that contains
+        # the start time of any of the associated observations.
+        if semester is None:
+            observation = self.observation_set.first()
+            if observation:
+                semester = get_semester_in(observation.start, observation.start)
+        # Fall back to using the semester that contains the first window start time.
+        if semester is None:
+            semester = get_semester_in(self.min_window_time, self.min_window_time)
+        return semester
 
     @property
     def time_allocation_keys(self):
@@ -318,7 +340,7 @@ class Request(models.Model):
             instrument_types__overlap=list({conf.instrument_type for conf in self.configurations.all()})
         )
 
-    def get_remaining_duration(self, configurations_after_priority, include_current=False):
+    def get_remaining_duration(self, configurations_after_priority, include_current=False, current_repeat=1):
         request_dict = self.as_dict()
         start_time = (min([window['start'] for window in request_dict['windows']])
                       if 'windows' in request_dict and request_dict['windows'] else timezone.now())
@@ -331,6 +353,14 @@ class Request(models.Model):
             start_time,
             configurations_after_priority
         )
+        if self.configuration_repeats > current_repeat:
+            # We have configuration repeats left, so add multiples of the total configuration duration
+            config_duration = get_total_complete_configurations_duration(
+                configurations,
+                start_time
+            )
+            repeats_left = self.configuration_repeats - current_repeat
+            duration += (repeats_left * config_duration)
         if include_current:
             previous_optical_elements = {}
             for configuration_dict in configurations:
@@ -461,7 +491,7 @@ class Configuration(models.Model):
                   'possible given its visibility within the observing window.'
     )
 
-    extra_params = JSONField(
+    extra_params = models.JSONField(
         default=dict,
         blank=True,
         verbose_name='extra parameters',
@@ -649,7 +679,7 @@ class Target(models.Model):
         help_text='Epoch of perihelion (MJD)'
     )
 
-    extra_params = JSONField(
+    extra_params = models.JSONField(
         default=dict,
         blank=True,
         verbose_name='extra parameters',
@@ -677,7 +707,7 @@ class InstrumentConfig(models.Model):
         Configuration, related_name='instrument_configs', on_delete=models.CASCADE,
         help_text='The Configuration to which this InstrumentConfig belongs'
     )
-    optical_elements = JSONField(
+    optical_elements = models.JSONField(
         default=dict,
         blank=True,
         help_text='Specification of optical elements used for this InstrumentConfig'
@@ -700,7 +730,7 @@ class InstrumentConfig(models.Model):
         help_text='(Spectrograph only) How the slit is positioned on the sky. If set to VFLOAT, atmospheric '
                   'dispersion is along the slit.'
     )
-    extra_params = JSONField(
+    extra_params = models.JSONField(
         default=dict,
         blank=True,
         verbose_name='extra parameters',
@@ -766,7 +796,7 @@ class GuidingConfig(models.Model):
         max_length=50, default='', blank=True,
         help_text='Guiding mode to use for the observations'
     )
-    optical_elements = JSONField(
+    optical_elements = models.JSONField(
         default=dict,
         blank=True,
         help_text='Optical Element specification for this GuidingConfig'
@@ -775,7 +805,7 @@ class GuidingConfig(models.Model):
         validators=[MinValueValidator(0.0), MaxValueValidator(120.0)],
         help_text='Guiding exposure time'
     )
-    extra_params = JSONField(
+    extra_params = models.JSONField(
         default=dict,
         blank=True,
         verbose_name='extra parameters',
@@ -805,7 +835,7 @@ class AcquisitionConfig(models.Model):
         validators=[MinValueValidator(0.0), MaxValueValidator(60.0)],
         help_text='Acquisition exposure time'
     )
-    extra_params = JSONField(
+    extra_params = models.JSONField(
         default=dict,
         blank=True,
         verbose_name='extra parameters',
@@ -839,8 +869,9 @@ class Constraints(models.Model):
         help_text='Minimum acceptable angular separation between the target and the moon in decimal degrees'
     )
     max_lunar_phase = models.FloatField(
-        verbose_name='Maximum lunar phase', null=True, blank=True,
-        help_text='Maximum acceptable lunar phase'
+        verbose_name='maximum lunar phase', default=1.0,
+        validators=[MinValueValidator(0.0), MaxValueValidator(1.0)],
+        help_text='Maximum acceptable lunar phase fraction from 0.0 (new moon) to 1.0 (full moon)'
     )
     max_seeing = models.FloatField(
         verbose_name='maximum seeing', null=True, blank=True,
@@ -850,7 +881,7 @@ class Constraints(models.Model):
         verbose_name='minimum transparency', null=True, blank=True,
         help_text='Minimum acceptable transparency'
     )
-    extra_params = JSONField(
+    extra_params = models.JSONField(
         default=dict,
         blank=True,
         verbose_name='extra parameters',
@@ -865,8 +896,9 @@ class Constraints(models.Model):
         return import_string(settings.AS_DICT['requestgroups']['Constraints'])(self)
 
     def __str__(self):
-        return 'Constraints {}: {} max airmass, {} min_lunar_distance'.format(self.id, self.max_airmass,
-                                                                              self.min_lunar_distance)
+        return 'Constraints {}: {} max airmass, {} min_lunar_distance, {} max_lunar_phase'.format(
+            self.id, self.max_airmass, self.min_lunar_distance, self.max_lunar_phase
+        )
 
 
 class DraftRequestGroup(models.Model):
