@@ -2,7 +2,7 @@ from django.utils import timezone
 from django.test import TestCase
 from mixer.backend.django import mixer
 from rest_framework.serializers import ValidationError
-from datetime import datetime
+from datetime import datetime, timedelta
 from unittest.mock import patch
 import math
 import copy
@@ -17,6 +17,7 @@ from observation_portal.common.test_helpers import SetTimeMixin
 from observation_portal.requestgroups.duration_utils import PER_CONFIGURATION_STARTUP_TIME
 from observation_portal.requestgroups.serializers import InstrumentTypeValidationHelper, ModeValidationHelper
 from observation_portal.requestgroups.test.test_api import generic_payload
+from observation_portal.observations.models import Observation
 
 
 class TestRequestGroupTotalDuration(SetTimeMixin, TestCase):
@@ -435,6 +436,68 @@ class TestRequestDuration(SetTimeMixin, TestCase):
 
         self.assertEqual(duration, (exp_s_duration + exp_a_duration + exp_l_duration + num_configurations*(PER_CONFIGURATION_STARTUP_TIME)))
 
+    def test_configuration_repeats_single_configuration_request_duration(self):
+        self.configuration_expose.request = self.request
+        self.configuration_expose.save()
+        self.request.configuration_repeats = 3
+        self.request.save()
+
+        self.instrument_config_expose.configuration = self.configuration_expose
+        self.instrument_config_expose.save()
+
+        duration = self.request.duration
+        exp_count = self.instrument_config_expose.exposure_count
+        exp_time = self.instrument_config_expose.exposure_time
+
+        configuration_duration = exp_count*(exp_time + self.sbig_readout_time2 + self.sbig_fixed_overhead_per_exposure) + self.sbig_filter_optical_element_change_overhead + PER_CONFIGURATION_STARTUP_TIME + self.instrument_change_overhead_1m + self.minimum_slew_time
+        self.assertEqual(duration, math.ceil( self.request.configuration_repeats * configuration_duration + self.sbig_front_padding))
+
+    def test_configuration_repeats_multi_configuration_request_duration(self):
+        self.configuration_expose.request = self.request
+        self.configuration_expose.save()
+        self.instrument_config_expose_1.configuration = self.configuration_expose
+        self.instrument_config_expose_1.save()
+        self.instrument_config_expose_2.configuration = self.configuration_expose
+        self.instrument_config_expose_2.save()
+        self.instrument_config_expose_3.configuration = self.configuration_expose
+        self.instrument_config_expose_3.save()
+
+        self.request.configuration_repeats = 3
+        self.request.save()
+
+        duration = self.request.duration
+
+        exp_time1 = self.instrument_config_expose_1.exposure_time
+        exp_count1 = self.instrument_config_expose_1.exposure_count
+        exp_time2 = self.instrument_config_expose_2.exposure_time
+        exp_count2 = self.instrument_config_expose_2.exposure_count
+        exp_time3 = self.instrument_config_expose_3.exposure_time
+        exp_count3 = self.instrument_config_expose_3.exposure_count
+
+        exp_1_duration = exp_count1*(exp_time1 + self.sbig_readout_time2 + self.sbig_fixed_overhead_per_exposure)
+        exp_2_duration = exp_count2*(exp_time2 + self.sbig_readout_time2 + self.sbig_fixed_overhead_per_exposure)
+        exp_3_duration = exp_count3*(exp_time3 + self.sbig_readout_time2 + self.sbig_fixed_overhead_per_exposure)
+
+        num_filter_changes = 2
+        configuration_duration = exp_1_duration + exp_2_duration + exp_3_duration + num_filter_changes*self.sbig_filter_optical_element_change_overhead + (PER_CONFIGURATION_STARTUP_TIME + self.minimum_slew_time)
+
+        self.assertEqual(duration, math.ceil(self.request.configuration_repeats * configuration_duration + self.sbig_front_padding))
+
+    def test_configuration_repeats_repeat_configuration_request_duration(self):
+        self.configuration_repeat_expose.request = self.request
+        self.configuration_repeat_expose.save()
+        self.request.configuration_repeats = 2
+        self.request.save()
+
+        configuration_duration = self.configuration_repeat_expose.repeat_duration
+        self.assertEqual(configuration_duration, self.configuration_repeat_expose.duration)
+
+        # Add the non-repeat parts back to the configuration duration
+        configuration_duration += + self.sbig_filter_optical_element_change_overhead + self.instrument_change_overhead_1m + self.minimum_slew_time
+
+        duration = self.request.duration
+        self.assertEqual(duration, math.ceil( self.request.configuration_repeats * configuration_duration + self.sbig_front_padding))
+
     def test_get_duration_from_non_existent_camera(self):
         self.configuration_expose.instrument_type = 'FAKE-CAMERA'
         self.configuration_expose.save()
@@ -544,3 +607,32 @@ class TestValidationHelper(TestCase):
         with self.assertRaises(ValidationError) as e:
             validation_helper.validate(instrument_config)
         self.assertIn('exposure_mode', str(e.exception))
+
+
+class TestRequestSemester(SetTimeMixin, TestCase):
+    def setUp(self) -> None:
+        super().setUp()
+        self.semester_1 = mixer.blend(Semester, start=datetime(2016, 1, 1, 0, 0, 0, tzinfo=timezone.utc), end=datetime(2016, 1, 31, 23, 59, 59, tzinfo=timezone.utc))
+        self.semester_2 = mixer.blend(Semester, start=datetime(2016, 2, 1, 0, 0, 0, tzinfo=timezone.utc), end=datetime(2016, 2, 29, 23, 59, 59, tzinfo=timezone.utc))
+        self.request = mixer.blend(Request)
+
+    def test_get_semester(self):
+        mixer.blend(Window, request=self.request, start=self.semester_1.start + timedelta(days=1), end=self.semester_1.end - timedelta(days=1))
+        mixer.blend(Window, request=self.request, start=self.semester_1.start + timedelta(days=2), end=self.semester_1.end - timedelta(days=2))
+        semester = self.request.semester
+        self.assertEqual(semester.id, self.semester_1.id)
+
+    def test_get_semester_for_request_without_observations_when_windows_span_multiple_semesters(self):
+        mixer.blend(Window, request=self.request, start=self.semester_1.start + timedelta(days=1), end=self.semester_1.end - timedelta(days=1))
+        mixer.blend(Window, request=self.request, start=self.semester_2.start - timedelta(days=1), end=self.semester_2.start + timedelta(days=1))
+        semester = self.request.semester
+        # Should fall into the semester with the earliest window start time
+        self.assertEqual(semester.id, self.semester_1.id)
+
+    def test_get_semester_for_request_with_observations_when_windows_span_multiple_semesters(self):
+        mixer.blend(Window, request=self.request, start=self.semester_1.start + timedelta(days=1), end=self.semester_1.end - timedelta(days=1))
+        mixer.blend(Window, request=self.request, start=self.semester_2.start - timedelta(days=1), end=self.semester_2.start + timedelta(days=1))
+        mixer.blend(Observation, request=self.request, start=self.semester_2.start + timedelta(hours=1), end=self.semester_2.start + timedelta(hours=2))
+        semester = self.request.semester
+        # Should fall into the semester that contains any observation
+        self.assertEqual(semester.id, self.semester_2.id)
